//  Java CGI -- Classes to aid in writing CGI programs in Java.
//
//  Copyright (c) 1996, David H. Silber  (dhs@orbits.com)
//
//    My time to write this free software donated by:
//                   Stellar Orbits Technology Services
//              http://www.orbits.com/   or   info@orbits.com
//
//    I can be contacted at:    1923 Rockland Avenue
//                              Rockville, MD  20851
//
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License version 2 as published by the Free Software Foundation.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

package Orbits.net;

import java.io.*;
import java.util.*;

import Orbits.text.Text;

//  This class will manage an e-mail message built from little bits.
//    Text will be collected as it is passed into this class.
//    When requested, the text will be formatted and sent.
public class Email extends Text {

	//  Debugging time?
	private boolean debug = false;

	//  Information for e-mail message header:
	Vector to = new Vector();	//  List of To: addresses.
	Vector cc = new Vector();	//  List of Cc: addresses.
	Vector bcc = new Vector();	//  List of Bcc: addresses.
	String subject;			//  Subject header text.

	//  Information for formatting the message:
	int column = 0;
	int rightMargin = 68;
	boolean lineOverFlow = false;


	//  Constructor for when we have no idea how much space to expect
	//    to use.
	public Email() {
		super();
	}


/*  Implement later:
	//  Constructor for when we do have an idea how much space to expect
	//    to use.
	public Email( int capacity ) {
		Text( int capacity );
	}
*/


	//  When it is time to break text between lines, scan backwards for
	//    the most recent whitespace character to replace with a NEWLINE.
	//  Return true if this line contains a word which is longer than the
	//    line should be.
	boolean lineBreak() {
		int length = output.length(); //  Get length of output buffer.
		int index;		//  The character we are working with.
		int leftMargin;		//  The limit to how far back to go.
		char c = NEWLINE;	//  The character we are working with.
		//  (If the first attempt to read fails, we have a reasonable
		//    default.)


		//  Back up through the text to the point where the left
		//    margin should be.
		for ( index = length - 1, leftMargin = index - column;
					index > leftMargin;  index-- ) {
			try {
				//  Get the character at this position.
				c = output.charAt( index );
			} catch( StringIndexOutOfBoundsException e ) {
				System.out.print( "Exception while reading " );
				System.out.println( "output buffer." );
			}

			//  If the current character is whitespace,
			if ( Character.isSpace( c ) ) {
				try {
					//  Replace whitespace with NEWLINE.
					output.setCharAt( index, NEWLINE );
				} catch( StringIndexOutOfBoundsException e ) {
					System.out.print( "Exception while " );
					System.out.print( "writing output " );
					System.out.println( "buffer." );
				}
				column = length - index - 1;

				//  Normal return.
				return false;
			}
		}
		//  If we have fallen through to here, we have a lineOverFlow
		//    condition.
		return true;
	}


	//  If special formatting marker is encountered, replace as per code.
	void formatSpecial( int index ) {
		switch ( text.charAt( index ) ) {

			//  A percent marker gets replaced with a percent,
			//    since the percent is being used as the marker
			//    character.
			case PERCENT:
				output.append( META );
				column++;
				break;

			//  A paragraph gets two newlines, a line break only
			//    gets one.
			case PARAGRAPH:
				output.append( NEWLINE );
			case LINEBREAK:
				output.append( NEWLINE );
				column = 0;
				lineOverFlow = false;
				break;

			//  We should never hit this.
			default:
				if ( debug ) {
					System.out.println( "" );
					System.out.print( "Email::formatSpec" );
					System.out.print( "ial:  Hit default" );
					System.out.println( "." );
					System.out.println( output.toString() );
					System.out.println( "" );
				}
				break;
		}
	}


	//  Format this Text item into an e-mail message.
	void format() {
		//  Don't reformat this text unless it has been changed.
		if ( formatted ) {
			return;
		}

		//  Scan through text buffer.
		for ( int index = 0;  index < text.length();  index++ ) {

			//  Get each character in turn.
			char c = text.charAt( index );

			if ( c == META ) {
				//  If this is the marker character, deal with
				//    special formatting.
				formatSpecial( ++index );
			}
			else if ( lineOverFlow && Character.isSpace( c ) ) {
				//  If during a lineOverFlow condition, a
				//    whitespace character is detected,
				//    break the line immediately and end the
				//    lineOverFlow status.
				output.append( NEWLINE );
				c = NEWLINE;  // So that column is set right.
				lineOverFlow = false;
			}
			else {
				//  Normally, each character is simply copied
				//    to the output buffer.
				output.append( c );
			}

			//  Keep track of the current column.
			if ( c == NEWLINE ) {
				column = 0;
				lineOverFlow = false;
			}
			else if ( c != META ) {
				column++;
			}

			//  When we hit the right margin, break the text.
			if ( column > rightMargin ) {
				lineOverFlow = lineBreak();
			}
		}

		//  Make sure that the last character of the output
		//    StringBuffer is a NEWLINE.
		if ( output.charAt( output.length() - 1 ) != NEWLINE ) {
			output.append( NEWLINE );
		}

		formatted = true;
	}


	//  Set the To: header.
	public void sendTo( String address ) {
		to.addElement( address );
	}


/*  Implement later:
	//  Set the To: header for multiple recipients.
	void sendTo( String [] address ) {
	}

	//  Set the Cc: header.
	void sendCc( String address ) {
	}

	//  Set the Cc: header for multiple recipients.
	void sendCc( String [] address ) {
	}

	//  Set the Bcc: header.
	void sendBcc( String address ) {
	}

	//  Set the Bcc: header for multiple recipients.
	void sendBcc( String [] address ) {
	}
*/


	//  Set the Subject: header.
	public void subject( String subject ) {
		this.subject = new String( subject );
	}


	//  Send the appropriate text to sendmail to complete the e-mail
	//    message.
	private void mailText( PrintStream out ) throws IOException {

		//  Create the list of To: addresses.
		Enumeration toList = to.elements();
		out.print( "To:" );
		while ( toList.hasMoreElements() ) {
			String toItem = (String) toList.nextElement();
			out.print( " " + toItem );
		}
		out.println( "" );

		//  Create the Subject: header.
		out.print( "Subject: " );
		out.println( subject );

		//  End the e-mail header with a blank line.
		out.println( "" );

		//  Dump the message contents.
		out.print( output.toString() );

		//  End the message text.
		out.println( "." );
	}


	//  Build the array of program name and arguments for executing
	//    sendmail.
	private String [] buildProgramArgs() {
		int counter = 0;

		//  programArgs contains enough space for all of the TO:
		//    addresses and the program name.
		String [] programArgs = new String [ to.size() + 1 ];

		//  We are making gross assumptions regarding the location
		//    of sendmail.  If this causes a problem send email to
		//    dhs@orbits.com describing the setup your system needs
		//    and I'll try to add that functionality.
		programArgs[ 0 ] = "/usr/lib/sendmail";

		//  Get the list of target addresses and add each to the
		//    sendmail arguments.
		Enumeration toList = to.elements();
		try {
			for ( counter = 1;  toList.hasMoreElements();
								counter++ ) {
				programArgs[ counter ] =
						(String) toList.nextElement();
			}
		} catch ( ArrayIndexOutOfBoundsException e ) {
			System.out.println( "Array error:" + e );
		}

		return ( programArgs );
	}


	//  Send the e-mail message.
	public void send() {
		//  Format the text collected thus far for this e-mail message.
		format();

		//  Get access to Runtime methods.
		Runtime rt = Runtime.getRuntime();

		//  Build the array of program name and arguments for exec.
		String [] programArgs = buildProgramArgs();

		try {
			//  Run sendmail.  Arguments are destination addresses.
			Process child = rt.exec( programArgs );

			//  Data for e-mail message contents it output to
			//    the standard input for sendmail.
			OutputStream child_out = child.getOutputStream();
			PrintStream out = new PrintStream( child_out );

			//  Send the e-mail message text to sendmail.
			mailText( out );

			//  Shut down the output stream.
			out.close();
		} catch ( IOException e ) {
			System.out.println( "Exec error:" + e );
		}
	}


	//  Extract the e-mail message, after formatting it.
	protected String read() {
		format();
		return ( output.toString() );
	}
}

