//  Java CGI -- Classes to aid in writing CGI programs in Java.
//
//  Copyright (c) 1996, David H. Silber  (dhs@orbits.com)
//
//    My time to write this free software donated by:
//                   Stellar Orbits Technology Services
//              http://www.orbits.com/   or   info@orbits.com
//
//    I can be contacted at:    1923 Rockland Avenue
//                              Rockville, MD  20851
//
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License version 2 as published by the Free Software Foundation.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

package Orbits.net;

import java.io.*;
import java.util.*;

import Orbits.text.Text;

//  This class will manage an HTML response message built from little bits.
//    Text will be collected as it is passed into this class.
//    When requested, the text will be formatted and sent.
public class HTML extends Text {

	//  Debugging time?
	private boolean debug = false;

	//  Information for HTML message:
	String title;			//  Title text.
	String author;			//  Name of author.

	//  Right now these suffice, as lists cannot be nested.  Later
	//    nesting should be provided for.
	int listType;	// Make this a stack later.
	int listStatus;	// Make this a stack later.
			//  One of LISTUNORDER, LISTTERM, LISTITEM, LISTEND.

	//  Predefined HTML code sequences with newlines to make the
	//    generated HTML source more readable.
	protected static final String HTMLparagraph      = "\n\n<p>\n";
	protected static final String HTMLlinebreak      = "<br>\n";
	protected static final String HTMLlistunorder    = "\n<ul>\n";
	protected static final String HTMLlistunorderend = "\n</ul>\n";
	protected static final String HTMLlistorder      = "\n<ol>\n";
	protected static final String HTMLlistorderend   = "\n</ol>\n";
	protected static final String HTMLlistdir        = "\n<dir>\n";
	protected static final String HTMLlistdirend     = "\n</dir>\n";
	protected static final String HTMLlistmenu       = "\n<menu>\n";
	protected static final String HTMLlistmenuend    = "\n</menu>\n";
	protected static final String HTMLlistdefine     = "\n<dl>\n";
	protected static final String HTMLlistdefineend  = "\n</dl>\n";
	protected static final String HTMLlistterm       = "\n<dt>\n";
	protected static final String HTMLlistitem       = "\n<li>\n";
	protected static final String HTMLlistitemdef    = "\n<dd>\n";


	//  Constructor for when we have no idea how much space to expect
	//    to use.
	public HTML() {
		super();
	}

/*  Implement later.
	//  Constructor for when we do have an idea how much space to expect
	//    to use.
	public HTML( int capacity ) {
		Text( int capacity );
	}
*/

	//  Set title of this document.
	public void title( String title ) {
		this.title = new String ( title );
	}


	//  Set author of this document.
	public void author( String author ) {
		this.author = new String ( author );
	}


	//  Start a definition list.
	public void definitionList() {

		//  Specify the type and new status for the list.
		listType = LISTDEFINE;
		listStatus = LISTUNORDER;

		//  Add the formatting code to start a definition list.
		text.append( META );
		text.append( LISTDEFINE );
	}


	//  Start a definition list term.
	public void definitionListTerm() {

		//  Specify the new status for the list.
		listStatus = LISTTERM;

		//  Add the formatting code to start the term part of a
		//    definition list entry.
		text.append( META );
		text.append( LISTTERM );
	}


	//  Start a list item.
	public void listItem() {

		//  Specify the new status for the list.
		listStatus = LISTITEM;

		//  Add the formatting code to start a list entry.
		text.append( LISTITEM );
		text.append( META );
	}


	//  Start a list item.
	public void listItem( String item ) {

		//  Specify the new status for the list.
		listStatus = LISTITEM;

		//  Add the formatting code to start a list entry and
		//    the text of the entry itself.
		text.append( META );
		text.append( LISTITEM );
		text.append( item );
	}


	//  All-in-one for definition terms and data items together:
	//    Return false if we are not in a definition list.
	public boolean listItem( String term, String item ) {
		if ( listType == LISTDEFINE ) {

			//  Specify the new status for the list.
			listStatus = LISTITEM;

			//  Add the formatting code and text for
			//    term part of entry.
			text.append( META );
			text.append( LISTTERM );
			text.append( term );

			//  Add the formatting code and text for
			//    definition part of entry.
			text.append( META );
			text.append( LISTITEM );
			text.append( item );

			//  In definition list, use of this method is OK.
			return true;
		}

		//  Not in definition list, use of this method is not OK.
		return false;
	}


	//  End any type of list.
	public void endList() {

		//  Specify the new status for the list.
		listType = LISTEND;
		listStatus = LISTEND;

		//  Add the formatting code for the end of a list.
		text.append( META );
		text.append( LISTEND );
	}


/*  Implement later.
	//  Start an unordered list.
	public void unorderedList() {
		listType = LISTUNORDER;
		text.append( META );
		text.append( LISTUNORDER );
	}

	//  Start an ordered list.
	public void orderedList() {
		listType = LISTORDER;
		text.append( META );
		text.append( LISTORDER );
	}

	//  Start a directory list.
	public void directoryList() {
		listType = LISTDIR;
		text.append( META );
		text.append( LISTDIR );
	}

	//  Start a menu list.
	public void menuList() {
		listType = LISTMENU;
		text.append( META );
		text.append( LISTMENU );
	}

	//  Specify an anchor.
	void anchor( String anchorName ) {
	}


	//  Specify a link.
	void link( String url, String text ) {
	}


	//  Specify an applet link.
	void applet( String url, String altText ) {
	}
*/


	//  Build the HTML prefix and header into the output buffer.
	void formatHeader() {

		//  HTML prefix.
		output.append( "Content-type: text/html\n\n" );

		//  HTML header.
		output.append( "<html>\n" );
		output.append( "<head>\n" );
		output.append( "<meta name=Author content=\"" );
		output.append( author );
		output.append( "\">\n" );
		output.append( "<title>\n" );
		output.append( title );
		output.append( "</title>\n" );
		output.append( "</head>\n" );
		output.append( "<body>\n" );
	}


	//  Append the HTML footer to the output buffer.
	void formatFooter() {
		output.append( "</body>\n" );
		output.append( "</html>\n" );
	}


	//  If special formatting marker is encountered, replace as per code.
	void formatSpecial( int index ) {
		switch ( text.charAt( index ) ) {

			//  A percent marker gets replaced with a percent
			//    character, since the percent character is
			//    being used as the META marker.
			case PERCENT:
				output.append( META );
				break;

			case PARAGRAPH:
				output.append( HTMLparagraph );
				break;

			case LINEBREAK:
				output.append( HTMLlinebreak );
				break;

			case LISTUNORDER:
				output.append( HTMLlistunorder );
				listType = LISTUNORDER;
				break;

			case LISTORDER:
				output.append( HTMLlistorder );
				listType = LISTORDER;
				break;

			case LISTDIR:
				output.append( HTMLlistdir );
				listType = LISTDIR;
				break;

			case LISTMENU:
				output.append( HTMLlistmenu );
				listType = LISTMENU;
				break;

			case LISTDEFINE:
				output.append( HTMLlistdefine );
				listType = LISTDEFINE;
				break;

			case LISTTERM:
				//  Should add error checking.
				output.append( HTMLlistterm );
				break;

			case LISTITEM:
				//  Should add error checking.

				//  Definition lists use a different marker
				//    for the definition part of an entry
				//    than the other lists use for an entry
				//    item.
				if ( listType == LISTDEFINE ) {
					output.append( HTMLlistitemdef );
				}
				else {
					output.append( HTMLlistitem );
				}
				break;

			case LISTEND:
				//  Should add error checking.

				//  The type of list end marker depends on the
				//    type of the current list.
				switch ( listType ) {
					case LISTUNORDER:
						output.append( HTMLlistunorderend );
						break;
					case LISTORDER:
						output.append( HTMLlistorderend );
						break;
					case LISTDIR:
						output.append( HTMLlistdirend );
						break;
					case LISTMENU:
						output.append( HTMLlistmenuend );
						break;
					case LISTDEFINE:
						output.append( HTMLlistdefineend );
						break;
					default:
						break;
				}
				break;

			//  We should never hit this.
			default:
				if ( debug ) {
					System.out.println( "" );
					System.out.print( "HTML::formatSpec" );
					System.out.print( "ial:  Hit default" );
					System.out.println( "." );
					System.out.println( output.toString() );
					System.out.println( "" );
				}
				break;
		}
	}


	//  Format this Text item into an e-mail message.
	void format() {
		//  Don't reformat this text unless it has been changed.
		if ( formatted ) {
			return;
		}

		//  Add HTML prefix and header to the output buffer.
		formatHeader();

		//  Scan through text buffer.
		for ( int index = 0;  index < text.length();  index++ ) {

			//  Get each character in turn.
			char c = text.charAt( index );

			if ( c == META ) {
				//  If this is the marker character, deal with
				//    special formatting.
				formatSpecial( ++index );
			}
			else {
				//  Normally, each character is simply copied
				//    to the output buffer.
				output.append( c );
			}
		}

		//  Add HTML footer to the output buffer.
		formatFooter();

		//  Make sure that the last character of the output
		//    StringBuffer is a NEWLINE.
		if ( output.charAt( output.length() - 1 ) != NEWLINE ) {
			output.append( NEWLINE );
		}

		//  Don't reformat this message unless it is modified.
		formatted = true;
	}


	//  Send the message after formatting it.
	public void send() {
		format();

		//  Drop message contents into standard output.
		System.out.print( output.toString() );
	}


	//  Extract message, after formatting it.
	protected String read() {
		format();
		return ( output.toString() );
	}
}

