//  Java CGI -- Classes to aid in writing CGI programs in Java.
//
//  Copyright (c) 1996, David H. Silber  (dhs@orbits.com)
//
//    My time to write this free software donated by:
//                   Stellar Orbits Technology Services
//              http://www.orbits.com/   or   info@orbits.com
//
//    I can be contacted at:    1923 Rockland Avenue
//                              Rockville, MD  20851
//
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License version 2 as published by the Free Software Foundation.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

package Orbits.text;

import java.io.*;
import java.util.*;


//  This class will manage an item of text built up from little bits.
//    Text will be collected as it is passed into this class.
//    This class is intended to be the superclass for the Email and HTML
//        classes.
public abstract class Text {

	//  Internal buffer for raw text and formatting marks.
	protected StringBuffer text;

	//  Internal buffer for formatted text.
	protected StringBuffer output;

	//  The special indication of a formatting mark.  Used in within
	//    the text buffer.
	protected static final char  META = '%';

	//  Ye olde, generic newline character.
	protected static final char  NEWLINE = '\n';

	//  Formatting codes for various purposes.
	protected static final char	PERCENT      =  1,
					PARAGRAPH    =  2,
					LINEBREAK    =  3,
					LISTUNORDER  =  4,
					LISTORDER    =  5,
					LISTDIR      =  6,
					LISTMENU     =  7,
					LISTDEFINE   =  8,
					LISTTERM     =  9,
					LISTITEM     = 10,
					LISTEND      = 11;

	//  Only reformat text if it needs it.
	protected boolean formatted = false;


	//  Constructor for when we have no idea how big to make the
	//    buffer.
	public Text() {

		//  Just start off the two buffers.
		text = new StringBuffer();
		output = new StringBuffer();
	}


/*  Implement later.
	//  Constructor for when we do have an idea how big to make the
	//    buffer.
	public Text( int capacity ) {
		text = new StringBuffer( int capacity );
		//  I don't know that anything needs to be done here...
	}
*/


	//  Add a character to the text item.
	public void add( char addition ) {
		formatted = false;		//  Mark as unformatted.
		text.append( addition );

		//  Special case:  If addition is the percent sign, (which
		//    is being used as the META marker) note that in the
		//    formatting code for later reconstruction.
		if ( addition == META ) {
			text.append( PERCENT );
		}
	}


	//  Add a String to the text item.
	public void add( String addition ) {
		formatted = false;		//  Mark as unformatted.

		//  Convert to a StringBuffer so that special processing in
		//    StringBuffer version of add() method can be used.
		StringBuffer buffy = new StringBuffer( addition );
		add( buffy );
	}


	//  Add a StringBuffer to the text item.
	public void add( StringBuffer addition ) {
		formatted = false;		//  Mark as unformatted.

		//  Work through addition, checking for percent characters.
		//    The percent sign is overloaded by the META formatting
		//    marker, therefore a percent sign in the text requires
		//    the PERCENT formatting code.
		for ( int index = 0;  index < addition.length();  index++ ) {
			if ( addition.charAt( index ) == META ) {
				addition.insert( ++index, PERCENT );
			}
		}

		text.append( addition );
	}


/*  Implement later.
	//  Add another Text object to this one.
	void add( Text textItem ) {
	}
*/

/*  Implement later.
	//  Add a fixed-format character to the text item.
	void addFixed( char addition ) {
	}

	//  Add a fixed-format String to the text item.
	void addFixed( String addition ) {
	}

	//  Add a fixed-format StringBuffer to the text item.
	void addFixed( StringBuffer addition ) {
	}
*/


	//  Mark the beginning of a paragraph.
	public void addParagraph() {
		formatted = false;		//  Mark as unformatted.

		//  Add in formatting code indicating the start of a
		//    paragraph.
		text.append( META );
		text.append( PARAGRAPH );
	}


	//  Mark a hard line break.
	public void addLineBreak() {
		formatted = false;		//  Mark as unformatted.

		//  Add in formatting code indicating a hard line break.
		text.append( META );
		text.append( LINEBREAK );
	}


/*  Implement later.
	//  Mark the beginning of a list.
	void addListStart( String addition ) {
	}

	//  Mark the beginning of a list item.
	void addListItem( StringBuffer addition ) {
	}
*/


/*
	//  Format the text into the output buffer.  This method is intended
	//    to be overridden, but it performs a minimum functionality in
	//    case it is not overridden.
	void format() {
		output = text;
		formatted = true;
	}
*/


	//  Subclasses will need to provide appropriate format() methods.
	abstract void format();


	//  Read out the accumulated text.
	protected String read() {
		format();
		return ( output.toString() );
	}
}

